"""Command-line interface."""
import click


@click.command()
@click.version_option()
def main() -> None:
    """Pyattck Data Models."""


if __name__ == "__main__":
    main(prog_name="pyattck-data-models")  # pragma: no cover
